/**
 * Copyrights (c) 2012 Advanced Core Technologies, Inc. All rights reserved.
 * 2012/05/10 ISHI 新規作成
 */
package jp.co.act_inc.actservicesample.adminservice;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import jp.co.act_inc.actservicesample.Logger;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlSerializer;

import android.net.Uri;
import android.util.Xml;

/**
 * ACT距離計算サービスのユーティリティです。
 * 
 * @author ishi
 * 
 */
public class ServiceUtils {
    /**
     * タイムアウト値（ミリ秒単位）
     */
    private static final int TIMEOUT = 180000;

    /**
     * HttpURLConnectionを作成します。
     * 
     * @param url
     *            URL
     * @param useSsl
     *            SSLを使用
     * @param soapAction
     *            SOAPアクション
     * @return 作成したHttpURLConnection
     * @throws IOException
     */
    public static HttpURLConnection createHttpURLConnection(String url,
            boolean useSsl, String soapAction) throws IOException {
        Uri uri = Uri.parse(url);
        Uri.Builder uriBuilder = new Uri.Builder();
        if (useSsl) {
            uriBuilder.scheme("https").authority(uri.getAuthority())
                    .path(uri.getPath()).query(uri.getQuery())
                    .fragment(uri.getFragment());

        } else {
            uriBuilder.scheme("http").authority(uri.getAuthority())
                    .path(uri.getPath()).query(uri.getQuery())
                    .fragment(uri.getFragment());
        }
        URL urlObject = new URL(uriBuilder.toString());
        HttpURLConnection connection = null;
        connection = (HttpURLConnection) urlObject.openConnection();
        connection.setRequestMethod("POST");
        connection.setDoOutput(true);
        connection.addRequestProperty("SOAPAction", soapAction);
        connection.addRequestProperty("Content-Type", "text/xml;charset=utf-8");
        connection.setConnectTimeout(TIMEOUT);
        connection.setReadTimeout(TIMEOUT);
        connection.setChunkedStreamingMode(0);

        return connection;
    }

    /**
     * サーバーからのレスポンスを取得します。
     * 
     * @param result
     *            計算結果
     * @param responseTag
     *            レスポンスのタグ
     * @param connection
     *            使用するHttpUrlConnection
     * @throws IOException
     * @throws XmlPullParserException
     * @throws WebException
     */
    public static <T extends Response> void getResponse(T result,
            String responseTag, HttpURLConnection connection)
            throws IOException, XmlPullParserException, WebException {
        if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
            InputStream stream = new BufferedInputStream(
                    connection.getInputStream());
            try {
                Logger.writeDebugLog("ActService", "parse start");
                XmlPullParser parser = Xml.newPullParser();
                parser.setInput(stream, "UTF-8");
                int eventType = parser.getEventType();
                while (eventType != XmlPullParser.END_DOCUMENT) {
                    switch (eventType) {
                    case XmlPullParser.START_TAG:
                        String startTagName = parser.getName();
                        if (startTagName.equalsIgnoreCase(responseTag)) {
                            result.readXml(parser, responseTag);
                            eventType = parser.next();
                            continue;
                        }
                        break;
                    }
                    eventType = parser.next();
                }
            } finally {
                stream.close();
            }
            Logger.writeDebugLog("ActService", "parse end");
        } else if (connection.getResponseCode() == HttpURLConnection.HTTP_INTERNAL_ERROR) {
            try {
                InputStream stream = connection.getInputStream();
                throwSoapException(stream);
                stream.close();
            } catch (Exception ex) {
                throw new WebException(connection.getResponseCode(),
                        connection.getResponseMessage());
            }
        } else {
            throw new WebException(connection.getResponseCode(),
                    connection.getResponseMessage());
        }
        return;
    }

    /**
     * SOAP例外を投げます。
     * 
     * @param stream
     *            計算結果が格納されたストリーム
     * @throws XmlPullParserException
     * @throws IOException
     * @throws SoapException
     */
    public static void throwSoapException(InputStream stream)
            throws XmlPullParserException, IOException, SoapException {
        try {
            XmlPullParser parser = Xml.newPullParser();
            parser.setInput(stream, "UTF-8");
            int eventType = parser.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) {
                switch (eventType) {
                case XmlPullParser.START_TAG:
                    String startTagName = parser.getName();
                    if (startTagName.equalsIgnoreCase("Fault")) {
                        throw SoapException.readXml(parser, "Falult");
                    }
                    break;
                }
                eventType = parser.next();
            }
        } catch (SoapException e) {
            throw e;
        }
    }

    /**
     * SOAPメッセージの末尾を書き込みます。
     * 
     * @param methodName
     *            メソッド名
     * @param xmlSerializer
     *            XMLシリアライザー
     * @throws IOException
     */
    public static void writeEndSoapMessage(String methodName,
            XmlSerializer xmlSerializer) throws IOException {
        xmlSerializer.endTag("", methodName);
        xmlSerializer.endTag("", "soap:Body");
        xmlSerializer.endTag("", "soap:Envelope");
        xmlSerializer.endDocument();
    }

    /**
     * SOAPメッセージの先頭を書き込みます。
     * 
     * @param methodName
     *            メソッド名
     * @param nameSpace
     *            名前空間
     * @param xmlSerializer
     *            XMLシリアライザー
     * @throws IOException
     */
    public static void writeStartSoapMessage(String methodName,
            String nameSpace, XmlSerializer xmlSerializer) throws IOException {
        xmlSerializer.startDocument("UTF-8", true);
        xmlSerializer.startTag("", "soap:Envelope");
        xmlSerializer.attribute("", "xmlns:soap",
                "http://schemas.xmlsoap.org/soap/envelope/");
        xmlSerializer.attribute("", "xmlns:xsi",
                "http://www.w3.org/2001/XMLSchema-instance");
        xmlSerializer.attribute("", "xmlns:xsd",
                "http://www.w3.org/2001/XMLSchema");
        xmlSerializer.startTag("", "soap:Body");
        xmlSerializer.startTag("", methodName);
        xmlSerializer.attribute("", "xmlns", nameSpace);
    }
}
